/* 
** Hashed linked list for local cache
** Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library and program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**   
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>

#include <libdav.h>
#include <davfsd.h>
#include <xml_result.h>
#include <xmlparse.h>

#define HASH_SIZE 256

/* data struct for hash*/
struct hash_list_t {
    long ino;            /* uniq fileid number */
    int wish;            /* open mode */
    int fp;              /* cache file handler */
    char *path;          /* cache file path */
    struct hash_list_t *next;
};

/* hash list */
struct hash_list_t *hash_list [HASH_SIZE];

/* add brand new entry */
int add_hash(long ino, int wish, int fp, char *path) {
    struct hash_list_t *new;
    
    new = malloc(sizeof(struct hash_list_t));
    if(!new) {
	perror("add_hash");
	return -1;
    }
    
    new->ino = ino;
    new->path = safe_strdup(path);
    new->wish = wish;
    new->fp = fp;
    
    new->next = hash_list[ino%HASH_SIZE];
    hash_list[ino%HASH_SIZE] = new; 
    
    return 0;
}

/* get information from hash and remove from the list */
int del_hash(long ino, int *wish, int *fp, char *path) {
    struct hash_list_t *prev, *list, *root = 
      hash_list[ino%HASH_SIZE];
   
    prev = NULL;

    for(list=root;list;list=list->next) {
	if(list->ino == ino) {
	    *wish = list->wish;
	    *fp   = list->fp;
	    strcpy(path, list->path);
	    safe_free(list->path);

	    if(list==root) {
		hash_list[ino%HASH_SIZE] = NULL;
	    } else if(prev) {
		prev->next = list->next;
	    }
	    safe_free(list);
	    return 1;
	}
	/* keep prev */
	prev = list;
    }
    
    return 0;
}

/* get chached file handelr */
int get_hash(long ino, int *fp) {
    struct hash_list_t *prev, *list, *root = 
      hash_list[ino%HASH_SIZE];
    
    prev = NULL;
    
    for(list=root;list;list=list->next) {
	if(list->ino == ino) {
	    *fp   = list->fp;
	    return 1;
	}
	/* keep prev */
	prev = list;
    }
    
    return 0;
}


/* set chached file handelr */
int set_hash(long ino, int fp) {
    struct hash_list_t *prev, *list, *root = 
      hash_list[ino%HASH_SIZE];
    
    prev = NULL;
    
    for(list=root;list;list=list->next) {
	if(list->ino == ino) {
	    list->fp = fp;
	    return 1;
	}
	/* keep prev */
	prev = list;
    }
    
    return 0;
}



