/* 
** umount all davfs  
** Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library and program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**   
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
*/

#include <stdio.h>
#include <string.h>
#include <mntent.h>
#include <sys/mount.h>

#include <davfsd.h>

int umount_all(void) {
    struct mntent *ment;
    FILE *mtab, *tmp_mtab;
    
    mtab = setmntent(MTAB, "r");
    tmp_mtab = setmntent(TMP_MTAB, "w");
    
    if (mtab == NULL) {
	fprintf(stderr, "ERROR: couldn't open "MTAB" for read\n");
	endmntent(mtab);
	endmntent(tmp_mtab);
	return 1;
    } else if (tmp_mtab == NULL) {
	fprintf(stderr, "ERROR: couldn't open "TMP_MTAB" for write\n");
	endmntent(mtab);
	endmntent(tmp_mtab);
	return 1;
    }
   
    /* umount all davfs */
    while((ment = getmntent(mtab)) != NULL) { 
	if(!strcmp(ment->mnt_type, "davfs")) {
	    umount(ment->mnt_dir);
	} else {
	    if (addmntent(tmp_mtab, ment) == 1) {
		fprintf(stderr, "ERROR: couldn't add entry to"TMP_MTAB"\n");
		endmntent(mtab);
		endmntent(tmp_mtab);
		return 1;
	    }
	    
	}
    }
    
    endmntent(mtab);
    endmntent(tmp_mtab);
    
    if (rename(TMP_MTAB, MTAB)) {
	fprintf(stderr, "ERROR: couldn't rename "TMP_MTAB" to "MTAB"\n");
	return 1;
    }
    
    return 0;
}

