/* 
** XML result library head file
** Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library and program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**   
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
*/

#ifndef __XML_RESULT_H__
#define __XML_RESULT_H__

/* max depth of XML */
#define MAX_DEPTH 256

/* XML attribute */
typedef struct xml_attr_t {
    char *name;            /* name */
    char *value;           /* value */
    struct xml_attr_t *next;
} xml_attr;

/* XML result */
typedef struct xml_result_t {
    int depth;             /* depth */     
    int order;             /* order in same depth */
    char *tag;             /* tag */
    char *data;            /* data */
    
    xml_attr *attr;        /* attribute */

    struct xml_result_t *parent;        /* parent */
    struct xml_result_t *first_child;   /* child */
    struct xml_result_t *next;          /* next */
    struct xml_result_t *depth_next;    /* next of same depth */
} xml_result; 

/* XML context handler */ 
typedef struct xml_ctx {
    int depth;            /* current depth */             
    xml_result root;      /* root result */
    xml_result *current;  /* current result */
    xml_result *depth_results[MAX_DEPTH]; /* depth root */
} xml_ctx;


/*
** prototype of unction
*/

/* XML parse handler */
void xr_data_handle(void *userdata, const char *data, int len); 
void xr_start(void *data, const char *el, const char **attr);
void xr_end(void *data, const char *el);

/* initialize and free */
void xr_init(xml_ctx *ctx);
void xr_free(xml_ctx *ctx);

/* find value using tags */
char *xr_find_data(xml_ctx *, char *tag);

/* find data in one bunch XML part */ 
char *xr_search_tag(xml_result *xr, char *tag);

/* get one bunce of XML in same depth */
xml_result *xr_one_result(xml_ctx *c, int depth);

/* conunt how many tags in same depth */
int xr_count(xml_ctx *c , int depth);

/* print out xml result */
int xr_print_friend(xml_result *res);

#endif


