/* 
** XML result library 
** Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library and program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**   
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <xmlparse.h>

#include <libdav.h>
#include <xml_result.h>

/* static functions */
static int xr_free_friend(xml_result *res);
static char * xr_search_friend(xml_result *res, char *tag); 

static void *xr_malloc(size_t size) {
    void *ptr = malloc(size);
    if(ptr) 
	memset(ptr, 0x00, size);
    return ptr;
}

void 
xr_data_handle(void *userdata, const char *data, int len) 
{
    xml_ctx *c = (xml_ctx *)userdata;
    char *ptr;

    // if(len==0) return;

    /* delete CR */
    // if(len==1 && *data==10) return;

    /* if already hava some data */
    if(c->current->data) { 
	ptr = xr_malloc(len+1+strlen(c->current->data));
	 
	if(c->current->data==NULL) {
	    perror("malloc_data_buff");
	    exit(-1);
	}

	strcpy(ptr, c->current->data);
    	safe_free(c->current->data);
	c->current->data = ptr;
    } else {
	c->current->data = xr_malloc(len+1);
	
	if(c->current->data==NULL) {
	    perror("malloc_data_buff");
	    exit(-1);
	}
    }

    /* we don't need to copy null 
       charater because of clear malloc */
    strncat(c->current->data, data, len);
    
#ifdef XML_DEBUG  
    printf("==> %s:%s(%d:%d)\n", c->current->tag, c->current->data, len, *data);
#endif
    return;
}

void
xr_start(void *data, const char *el, const char **attr) {
    int i;
    xml_attr *xr_attr;
    xml_ctx *c = (xml_ctx *) data;
    xml_result *dr, *xr = xr_malloc(sizeof(xml_result));
    
    if(xr==NULL) {
	perror("xml_result_malloc");
	exit(-1);
    }

    if(c->current==NULL) {
	perror("xml_ctx_not_initialized");
	exit(-1);
    }

    /* increment depth */
    xr->depth = ++c->depth;
    
    /* add depth_result */
    for(dr=c->depth_results[c->depth];
	dr && dr->depth_next;dr=dr->depth_next);
    
    /* first node */
    if(dr==NULL) 
	c->depth_results[c->depth] = xr;
    else dr->depth_next = xr; 
    
    /* check is child or friend */
    if(c->current->depth == xr->depth) {
	/* friend */
	xr->order = c->current->order+1;   
	xr->parent = c->current->parent;
	c->current->next = xr;
    } else if(c->current->depth < xr->depth ) {
	/* child */
	if(c->current->first_child==NULL) {
	    c->current->first_child = xr;
	}
	xr->parent = c->current;
    } else {
	if(xr->depth>SEP_XML)
	    c->current->parent->next = xr;

	xr->parent = c->current->parent->parent;
	/* how can we find the parent */
/*
	for(dr=c->current, i=0; i<c->current->depth-xr->depth; i++) {
	    dr = dr->parent;
	    if(dr==NULL) break;
	}
	if(dr) xr->parent = dr->parent;
*/

	/* link with children */
/*
	for(;dr->next;dr=dr->next);
	dr->next = xr;	
*/
    }
    
    c->current = xr;

    c->current->tag = safe_strdup(el);

#ifdef XML_DEBUG
	printf("START XML TAG : %s(%d)\n", c->current->tag, xr->depth);
#endif
    
    for (i = 0; attr[i]; i += 2) {
	if(i==0) { // first node
	    xr_attr = c->current->attr = xr_malloc(sizeof(xml_attr));
	} else {
	    xr_attr->next  = xr_malloc(sizeof(xml_attr));
	    xr_attr = xr_attr->next;
	}
	
	if(xr_attr==NULL) {
	    perror("xml_attr_malloc");
	    exit(-1);
	}
	xr_attr->name  = safe_strdup(attr[i]);
	xr_attr->value = safe_strdup(attr[i+1]); 
#ifdef XML_DEBUG
	printf(" %s='%s'", xr_attr->name, xr_attr->value);
#endif
    }
    
    return;
}  /* End of start handler */

void
xr_end(void *data, const char *el) {
    xml_ctx *c = (xml_ctx *)data;

    c->depth--;

    return;
}  /* End of end handler */

void
xr_init(xml_ctx *ctx) {
    memset(ctx, 0x00, sizeof(xml_ctx));

    ctx->depth_results[0] = ctx->current = &ctx->root;
    
    return;
}

int 
xr_print_child(xml_result *res) {
    int i;

    for(i=0;res;res=res->first_child, i++) {
	printf("[%d %s-%s]\n", res->depth, res->tag, res->data?res->data:"");
	xr_print_friend(res->next);
    }
    
    return i;
}

int 
xr_print_friend(xml_result *res) {
    int i;
    for(i=0;res;res=res->next, i++) {
	printf("[%d %s-%s]\n", res->depth, res->tag, res->data?res->data:"");
	xr_print_child(res->first_child);
    }
    return i;
}

void
xr_print(xml_ctx *ctx) {
    printf("ROOT");
    
    xr_print_child(ctx->root.first_child);
}
 
static int xr_free_attr(xml_attr *attr) {
    xml_attr *to_free;
    int i;

    for(i=0; attr; i++) {
	to_free = attr;
	attr=attr->next;
		
	safe_free(to_free->name);
 	to_free->name = NULL;	
	safe_free(to_free->value);
	to_free->value = NULL;
	safe_free(to_free);
	to_free = NULL;
    }

    return i;
}

static int xr_node_free(xml_result *node) {
    safe_free(node->tag);
    node->tag = NULL;
    safe_free(node->data);
    node->data = NULL;
    
    /* free attributes */
    xr_free_attr(node->attr);
    node->attr = NULL;
   
    return 0;
}

static int
xr_free_child(xml_result *res) {
    int i;
    xml_result *to_free;

    for(i=0;res;i++) {
	/* free his next node */
	xr_free_friend(res->next);
	
	to_free = res;
	res=res->first_child;

	/* free member variables */
	xr_node_free(to_free);

	safe_free(to_free);
	to_free = NULL;
    }
    
    return i;
}

static int
xr_free_friend(xml_result *res) {
    int i;
    xml_result *to_free;
    for(i=0; res; i++) {
	/* free his child node */
	xr_free_child(res->first_child);
	to_free = res;
	res=res->next;

	/* free member variables */
	xr_node_free(to_free);
	
	safe_free(to_free);
	to_free = NULL;
    }
    
    return i;
}

void
xr_free(xml_ctx *ctx) {
    /* free */
    xr_free_child(ctx->root.first_child);
    memset(&ctx, 0x00, sizeof(*ctx));
}
 

static int slash_count(const char *tags) {
    int ret=0;
    for(;*tags;tags++) {
	if(*tags=='/') ret++;
    }

    return ret;
}


/*
** c : xml_ctx 
**     NULL for next search
** tags : tags names seperated by /
** ex) '/response'
**     '/response/href'
**     '/response/prof/java
**
*/
char *xr_find_data(xml_ctx *c, char *tags) {
    char *ret, *ptr;
    char *buff, *one_tags[MAX_DEPTH];
    int i,match=1;
    xml_result *parent;
    static xml_result *keep;
    int depth = slash_count(tags);
    
    if(depth==0) return NULL;

    if(c) keep = c->depth_results[depth];
    
    if(keep==NULL) return NULL;

    /* seperate tags */
    memset(one_tags, 0x00, sizeof(one_tags));
    buff = safe_strdup(tags);
    one_tags[1] = strtok(buff, "/");
    
    for(i=2;i<MAX_DEPTH;i++) {
	one_tags[i] = strtok(NULL, "/");
	if(one_tags[i]==NULL) break;
    }

    for(;keep;keep=keep->depth_next) {
	match = 1;
	for(parent=keep, i=depth;i>0 && parent;i--, parent=parent->parent) {
	    if(one_tags[i]==NULL) {
		match = 0;
		break;
	    }
	    /* ignore name space */
	    if((ptr=strchr(parent->tag,':'))) 
		ptr++;
	    else 
		ptr = parent->tag;

	    if(strcmp(one_tags[i], ptr)) {
		match = 0;
		break;
	    }
	}
	 
	if(match) {
	    ret = keep->data;
	    keep = keep->depth_next;
	    safe_free(buff);
	    
	    return ret?ret:"";
	}
    }

    safe_free(buff);
    return NULL;
}

/*
xml_attr *xr_find_attr(xml_ctx *c, char *tags) {
    static xml_result *keep;

    if(c!=NULL) {
	keep = c->root.first_child;
    }

    if(keep==NULL) return NULL;
    
}
*/


/*
** return the count of depth
** you can use get result count of a certain depth
**
*/
int xr_count(xml_ctx *c , int depth) {
    xml_result *keep;
    int count;

    if(depth==0) return 0;
    
    keep = c->depth_results[depth];
    
    if(keep==NULL) return 0;

    for(count=0;keep;keep=keep->depth_next, count++)
	;

    return count;
}

/*
** get one result of a certain depth
**     NULL depth  for next search
** 
*/
xml_result *xr_one_result(xml_ctx *c, int depth) {
    static xml_result *keep;
    xml_result *ret;

    if(c) keep = c->depth_results[depth];

    /* nothing to return */
    if(keep==NULL) return NULL;

    ret = keep;
    keep = keep->depth_next;
    
    return ret;
}

/* 
** elemant xmlns part
*/
static int xr_strcmp(const char *tag, const char *key) {
    const char *ptr = strchr(tag, ':');
    
    if(ptr) 
	ptr++;
    else 
	ptr = tag;

    return strcmp(ptr, key);
}


static char * 
xr_search_child(xml_result *res, char *tag) {
    int i;
    char *ret = NULL;

    for(i=0;res;res=res->first_child, i++) {
	/* if find */
	if(!xr_strcmp(res->tag, tag)) 
	    return  res->data?res->data:"";
	else {
	    ret = xr_search_friend(res->next, tag);
	    if(ret)
		return ret;
	}
    }
    return NULL;
}

static char * 
xr_search_friend(xml_result *res, char *tag) {
    int i;
    char *ret = NULL;

    for(i=0;res;res=res->next, i++) {
	/* if find */
	if(!xr_strcmp(res->tag, tag)) 
	    return res->data?res->data:"";
	else { 
	    ret = xr_search_child(res->first_child, tag);
	    if(ret)
		return ret;
	}
    }
    return NULL;
}

/*
**
** get result in xml_result
**
*/
char *xr_search_tag(xml_result *res, char *tag) {
    return xr_search_friend(res, tag); 
}











