/* 
** Check dav server before mount
** Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library and program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**   
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <config.h>
#include <libdav.h>

/*
** Check server using given url, id and passwd.
** 
** return 1 on sucess
**
*/

int check_dav_server(char *url, char *id, char *pass,char *proxy)
{
  dav_config dc;
  dav_rec rec;
  int ret;

  dav_rec_clear(&rec);

  /* clear dc structure */
  memset(&dc, 0x00, sizeof(dav_config));

  /* open connection */
  if (dav_open(&dc, url, id, pass,strlen(proxy)?proxy:NULL) == NULL) {
    perror("dav_open");
    return 0;
  }

  ret = dav_options(&dc, &rec);
  dav_close(&dc);
  
  /* Check ret code */
  if (ret == 401) {
    fprintf(stderr, "Authorization Required(%d)\n", ret);
  } else if (ret != 200) {
    fprintf(stderr, "Bad response from server(%d)\n", ret);
  } else if (!rec.hh.allow) {
    fprintf(stderr, "No Allow head, Go ahead!!\n");
  } else if (!strstr(rec.hh.allow, "PROPFIND")) {
    fprintf(stderr, "Your server support only %s methods\n", rec.hh.allow);
    ret = 504;
  }

  dav_rec_free(&rec);

  return ret;
}
