#ifndef _PVFS_MOUNT_H
#define _PVFS_MOUNT_H

/*
 * copyright (c) 1999 Rob Ross and Phil Carns, all rights reserved.
 *
 * Written by Rob Ross and Phil Carns.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Contact:  Rob Ross    rbross@parl.clemson.edu
 *           Phil Carns  pcarns@parl.clemson.edu
 */

#include <linux/types.h>

#ifndef __KERNEL__
#include <netinet/in.h>
#endif

/* data structures */
enum
{
  DAV_HOSTLEN = 1024,
  DAV_PATHLEN = 1024,
  DAV_IDLEN = 80,
  DAV_PASSLEN = 80,
};

/*
 * Data from mount.davfs command
 *
 * You should keep same copy of kernel module
 */
struct dav_mount_data
{
  unsigned info_magic;
  int flags;
  int port;
  char host[DAV_HOSTLEN];
  char path[DAV_PATHLEN];
  char id[DAV_IDLEN];
  char pass[DAV_PASSLEN];
  char mpoint[DAV_PATHLEN];
  /* 'r' or 'rw' */
  char amode[3];
  /* socket */

  unsigned int dav_fd;

  /* mount demonized pid */
  long pid;

  /* user id ang group id */
  umode_t         mode;
  uid_t           uid;
  gid_t           gid;
   
  /* SSL mode */
  int ssl_mode;

 /* for Proxy */
  char proxy[DAV_HOSTLEN];	
};


#define MTAB "/etc/mtab"
#define TMP_MTAB "/etc/mtab.pvfs"

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 */

#define DEBUG 0
#define dprintf(args...) if (DEBUG) printf(args);

int usage(int argc, char **argv);
int do_mtab(struct mntent *);
int setup_mntent(struct mntent *, const char *, const char *, const char *,
		 const char *, int, int);

int parse_args(int argc, char *argv[], struct dav_mount_data *);
int talktodavfsd(struct dav_mount_data *dav_data,char *url);

int ResolveAddr(char *, struct in_addr *);
int run_davfsd(void);
int check_dav_server(char *url, char *id, char *pass, char *proxy);

#define DAVFSD_HOST "localhost"
#define DAVFSD_PORT 5555


#define DEBUG2(x,y)

#define sys_fork fork

#endif
